﻿using Newtonsoft.Json;
using System;
using System.Collections.Generic;
using System.ComponentModel.DataAnnotations;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace IRaCIS.Core.Application.Contracts
{

    public class UnionStudyBaseModel
    {
        public Guid TrialId { get; set; }

        public Guid SiteId { get; set; }

        public Guid SubjectId { get; set; }

        public Guid SubjectVisitId { get; set; }


        public string SubjectCode { get; set; }

        public string VisitName { get; set; } = string.Empty;

        public decimal VisitNum { get; set; }

        public string TrialSiteCode { get; set; } = string.Empty;

        public string TrialSiteAliasName { get; set; } = string.Empty;

        public string Uploader { get; set; } = string.Empty;

        public DateTime UploadTime { get; set; }

        public string StudyCode => IsDicom ? DicomStudyCode : "NST" + NoneDicomCode.ToString("D5");


        [JsonIgnore]
        public string DicomStudyCode { get; set; } = string.Empty;
        [JsonIgnore]
        public int NoneDicomCode { get; set; }

        public bool IsDicom { get; set; }
    }

    public class UnionStudyMonitorModel : UnionStudyBaseModel
    {
        public Guid StudyId { get; set; }


        public DateTime UploadStartTime { get; set; }


        public DateTime UploadFinishedTime { get; set; }


        public decimal FileSize { get; set; }

        public string IP { get; set; }


        public bool IsDicomReUpload { get; set; }

        public bool IsDicom { get; set; }

        public int FileCount { get; set; }

    }


    public class UnionStudyViewModel:UnionStudyBaseModel
    {
        
        public Guid Id { get; set; }

        

        public int? Count { get; set; }


        public string Modalities { get; set; } = string.Empty;

        public string Bodypart { get; set; } = string.Empty;

        public DateTime? StudyTime { get; set; }

       
    }



    public class StudyQuery:PageInput
    {
        [NotDefault]
        public Guid TrialId { get; set; }

        public Guid? SubjectId { get; set; }

        public Guid? SiteId { get; set; }
        public Guid? SubjectVisitId { get; set; }

        public string SubjectInfo { get; set; } = String.Empty;

        public string VisitPlanInfo { get; set; } = String.Empty;

    }
}
